package com.diagbot.aggregate;

import com.diagbot.dto.QcResultPercentDTO;
import com.diagbot.dto.ResultDetailDTO;
import com.diagbot.facade.BehospitalInfoFacade;
import com.diagbot.facade.QcresultInfoFacade;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.FilterVO;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:科室统计
 * @Author:zhaops
 * @time: 2020/5/6 17:21
 */
@Component
public class DeptStatisticsAggregate {
    @Autowired
    private BehospitalInfoFacade behospitalInfoFacade;
    @Autowired
    private QcresultInfoFacade qcresultInfoFacade;

    @DataProvider("setAllDept")
    public Map<String, Object> setAllDept(
            @InvokeParameter("filterVO") FilterVO filterVO,
            @DataConsumer("getLevelResultDept") List<QcResultPercentDTO> levelResults,
            @DataConsumer("getResultDept") List<ResultDetailDTO> results) {
        Map<String, Object> retMap = new LinkedHashMap<>();
        if (ListUtil.isNotEmpty(levelResults)) {
            retMap.put("各科室甲级病历占比", levelResults);
        }
        if (ListUtil.isNotEmpty(results)) {
            retMap.put("各科室缺陷占比", results);
        }
        return retMap;
    }

    @DataProvider("getLevelResultDept")
    public List<QcResultPercentDTO> getLevelResultDept(@InvokeParameter("filterVO") FilterVO filterVO) {
        List<QcResultPercentDTO> qcResultPercentList = qcresultInfoFacade.levelPercentGroupByDept(filterVO);
        qcResultPercentList = qcResultPercentList.stream().limit(10).collect(Collectors.toList());
        return qcResultPercentList;
    }


    @DataProvider("getResultDept")
    public List<ResultDetailDTO> getResultDept(@InvokeParameter("filterVO") FilterVO filterVO) {
        List<ResultDetailDTO> results2 = behospitalInfoFacade.resultStatisticsByDept2(filterVO);
        if (ListUtil.isNotEmpty(results2)) {
            int totle = results2
                    .stream()
                    .map(ResultDetailDTO::getNum)
                    .reduce(0, Integer::sum);
            results2.forEach(result -> {
                Double percent = BigDecimal.valueOf(result.getNum())
                        .divide(BigDecimal.valueOf(totle), 4, RoundingMode.HALF_UP)
                        .doubleValue();
                result.setPercent(percent);
            });
            List<ResultDetailDTO> retResults = Lists.newLinkedList();
            if (results2.size() <= 6) {
                retResults = BeanUtil.listCopyTo(results2, ResultDetailDTO.class);
            } else {

                Double rate = 0d;
                Integer num = 0;
                for (ResultDetailDTO result : results2) {
                    if (retResults.size() < 5) {
                        rate = BigDecimal.valueOf(rate)
                                .add(BigDecimal.valueOf(Double.valueOf(result.getPercent())))
                                .doubleValue();
                        retResults.add(result);
                    } else {
                        num += result.getNum();
                    }
                }
                ResultDetailDTO retResult = new ResultDetailDTO();
                retResult.setName("其他");
                retResult.setNum(num);
                retResult.setPercent(BigDecimal.valueOf(1).subtract(BigDecimal.valueOf(rate)).doubleValue());
                retResults.add(retResult);
            }
            retResults.forEach(result -> {
                DecimalFormat df = new DecimalFormat("#0.00");
                String percentStr
                        = df.format(BigDecimal.valueOf(result.getPercent()).multiply(BigDecimal.valueOf(100))) + "%";
                result.setPercentStr(percentStr);
            });
            return retResults;
        }
        return null;
    }
}
