package com.diagbot.aggregate;

import com.diagbot.dto.ResultDetailDTO;
import com.diagbot.facade.BehospitalInfoFacade;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.FilterVO;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:缺陷相关统计
 * @author: gaodm
 * @time: 2020/4/22 18:09
 */
@Component
public class ResultStatisticsAggregate {
    @Autowired
    private BehospitalInfoFacade behospitalInfoFacade;

    @DataProvider("setAllResult")
    public Map<String, Object> setAllResult(
            @InvokeParameter("filterVO") FilterVO filterVO,
            @DataConsumer("getResult") List<ResultDetailDTO> results) {
        Map<String, Object> retMap = new LinkedHashMap<>();
        if (ListUtil.isNotEmpty(results)) {
            retMap.put("缺陷排行列表", results);
        }

        return retMap;
    }

    @DataProvider("getResult")
    public List<ResultDetailDTO> getResult(@InvokeParameter("filterVO") FilterVO filterVO) {
        List<ResultDetailDTO> results = behospitalInfoFacade.resultStatistics2(filterVO);

        if (ListUtil.isNotEmpty(results)) {
            int totle = results
                    .stream()
                    .map(ResultDetailDTO::getNum)
                    .reduce(0, Integer::sum);
            List<ResultDetailDTO> retResutls = Lists.newLinkedList();
            results.forEach(result -> {
                Double percent = BigDecimal.valueOf(result.getNum())
                        .divide(BigDecimal.valueOf(totle), 4, RoundingMode.HALF_UP)
                        .doubleValue();
                result.setPercent(percent);
                DecimalFormat df = new DecimalFormat("#0.00");
                String percentStr
                        = df.format(BigDecimal.valueOf(result.getPercent()).multiply(BigDecimal.valueOf(100))) + "%";
                result.setPercentStr(percentStr);
                if (retResutls.size() < 10) {
                    retResutls.add(result);
                }

            });
            return retResutls;
        }

        return null;
    }

}
